@@ -51,7 +51,7 @@ sub DESTROY { }
 
 package Curses;
 
-$VERSION = '1.27'; # Makefile.PL picks this up
+$VERSION = '1.29'; # Makefile.PL picks this up
 
 use Carp;
 require Exporter;
@@ -92,6 +92,8 @@ tie $COLOR_PAIRS, Curses::Vars, 6;
     LINES $LINES COLS $COLS stdscr $stdscr curscr $curscr COLORS $COLORS
     COLOR_PAIRS $COLOR_PAIRS
 
+    getchar getstring ungetchar instring addstring insstring
+
     addch echochar addchstr addchnstr addstr addnstr attroff attron attrset
     standend standout attr_get attr_off attr_on attr_set chgat COLOR_PAIR
     PAIR_NUMBER beep flash bkgd bkgdset getbkgd border box hline vline
@@ -254,10 +256,6 @@ Curses - terminal screen handling and optimization
     ...
     endwin;
 
-
-   Curses::supports_function($function);
-   Curses::supports_constant($constant);
-
 =head1 DESCRIPTION
 
 C<Curses> is the interface between Perl and your system's curses(3)
@@ -311,6 +309,161 @@ variable C<$Curses::OldCurses> to a non-zero value before using the
 C<Curses> package.  See L<"Perl 4.X C<cursperl> Compatibility">
 for an example of this.
 
+=head2 Wide-Character-Aware Functions
+
+The following are the preferred functions for working with strings, though
+they don't follow the normal unified function naming convention (based on the
+names in the Curses library) described above.  Despite the naming, each
+corresponds to a Curses library function.  For example, a C<getchar>
+call performs a Curses library function in the C<getch> family.
+
+In addition to these functions, The C<Curses> module contains corresponding
+functions with the conventional naming (e.g. C<getch>); the duplication is for
+historical reasons.  The preferred functions were new in Curses 1.29 (April
+2014).  They use the wide character functions in the Curses library if
+available (falling back to using the traditional non-wide-character versions).
+They also have a more Perl-like interface, taking care of some gory details
+under the hood about which a Perl programmer shouldn't have to worry.
+
+The reason for two sets of string-handling functions is historical.  The
+original Curses Perl module predates Curses libraries that understand wide
+(multibyte) characters.  Moreover, the module was designed to have a Perl
+interface that closely resembles the C interface syntactically and directly
+passes the internal byte representation of Perl strings to C code.  This was
+probably fine before Perl got Unicode function, but today, Perl stores strings
+internally in either Latin-1 or UTF-8 and the original module was not
+sensitive to which encoding was used.
+
+While most of the problems could be worked around in Perl code using the
+traditional interface, it's hard to get right and you need a
+wide-character-aware curses library (e.g. ncursesw) anyway to make it work
+properly.  Because existing consumers of the Curses module may be relying on
+the traditional behavior, Curses module designers couldn't simply modify the
+existing functions to understand wide characters and convert from and to Perl
+strings.
+
+=head3 C<getchar>
+
+This calls C<wget_wch()>.  It returns a character -- more precisely, a
+one-character (not necessarily one-byte!) string holding the character -- for
+a normal key and a two-element list C<(undef, key-number)> for a function key.
+It returns C<undef> on error.
+
+If you don't expect function keys (i.e. with C<keypad(0))>, you can simply do
+
+=over 4
+
+	my $ch = getchar;
+	die "getchar failed" unless defined $ch;
+
+=back
+
+If you do expect function keys (i.e. with C<keypad(1)>), you can still assign
+the result to a scalar variable as above.  Because of of the way the comma
+operator works, that variable will receive either C<undef> or the string or
+the number, and you can decode it yourself.
+
+=over 4
+
+	my $ch = getchar;
+	die "getchar failed" unless defined $ch;
+	if (<$ch looks like a number >= 0x100>) {
+		<handle function key>
+	} else {
+		<handle normal key>
+	}
+
+=back
+
+or do
+
+=over 4
+
+	my ($ch, $key) = getchar;
+	if (defined $key) {
+		<handle function key $key>
+	} else if (defined $ch) {
+		<handle normal key $ch>
+	} else {
+		die "getchar failed";
+	}
+
+=back
+
+If C<wget_wch()> is not available (i.e. The Curses library does not understand
+wide characters), this calls C<wgetch()>, but returns the values described
+above nonetheless.  This can be a problem because with a multibyte character
+set like UTF-8, you will receive two one-character strings for a
+two-byte-character (e.g. "" and "" for "").  If you append
+these characters to a Perl string, that string may internally contain a valid
+UTF-8 multibyte character, but Perl will not interpret it that way. Perl may
+even try to convert what it believes to be two characters to UTF-8, giving you
+four bytes.
+
+
+=head3 C<getstring>
+
+This calls C<wgetn_wstr> and returns a string or C<undef>.  It cannot return a
+function key value; the Curses library will itself interpret KEY_LEFT and
+KEY_BACKSPACE.
+
+If C<wget_wch()> is unavailable, this calls C<wgetstr()>.
+
+In both cases, the function allocates a buffer of fixed size to hold the
+result of the Curses library call.
+
+=over 4
+
+	my $s = getstring();
+	die "getstring failed" unless defined $s;
+
+=back
+
+
+=head3 C<addstring>/C<insstring>
+
+This adds/inserts the Perl string passed as an argument to the Curses window
+using C<waddnwstr()>/C<wins_nwstr()> or, if unavailable,
+C<waddnstr()>/C<winsnstr()>.  It returns a true value on success, false on
+failure.
+
+=over 4
+
+	addstring("Hll, Wrld") ||die "addstring failed";
+
+=back
+
+=head3 C<instring>
+
+This returns a Perl string (or C<undef> on failure) holding the characters
+from the current cursor position up to the end of the line.  It uses
+C<winnwstr()> if available, and otherwise C<innstr()>.
+
+=over 4
+
+	my $s = instring();
+	die "instring failed" unless defined $s;
+
+=back
+
+=head3 C<ungetchar>
+
+This pushes one character (passed as a one-character Perl string) back to the
+input queue. It uses C<unget_wch()> or C<ungetch()>.  It returns a true value
+on success, false on failure.  It cannot push back a function key; the Curses
+library provides no way to push back function keys, only characters.
+
+=over 4
+
+	ungetchar("X") ||die "ungetchar failed";
+
+=back
+
+The C<Curses> module provides no interface to the complex-character routines
+(C<wadd_wch()>, C<wadd_wchnstr()>, C<wecho_wchar()>, C<win_wch()>,
+C<win_wchnstr()>, C<wins_wch()>) because there is no sensible way of
+converting from Perl to a C C<cchar_t> or back.
+
 =head2 Objects
 
 Objects work.  Example:
@@ -346,45 +499,38 @@ In order to avoid this problem, use the alternate functions:
    inchnstr()
    innstr()
 
-which take an extra "size of buffer" argument.
+which take an extra "size of buffer" argument
+or the wide-character-aware getstring() and instring() versions.
 
 =head1 COMPATIBILITY
 
 =head2 Perl 4.X C<cursperl> Compatibility
 
-C<Curses> has been written to take advantage of the new features of
-Perl.  I felt it better to provide an improved curses programming
-environment rather than to be 100% compatible.  However, many old
-C<curseperl> applications will probably still work by starting the
-script with:
+C<Curses> was written to take advantage of features of Perl 5 and later.  The
+author thought it was better to provide an improved curses programming
+environment than to be 100% compatible.  However, many old C<curseperl>
+applications will probably still work by starting the script with:
 
     BEGIN { $Curses::OldCurses = 1; }
     use Curses;
 
-Any old application that still does not work should print an
-understandable error message explaining the problem.
+Any old application that still does not work should print an understandable
+error message explaining the problem.
 
 Some functions and variables are not available through C<Curses>, even with
 the C<BEGIN> line.  They are listed under
-L<"curses(3) items not available through Curses">.
+L<"Curses items not available through Perl Curses">.
 
 The variables C<$stdscr> and C<$curscr> are also available as
 functions C<stdscr> and C<curscr>.  This is because of a Perl bug.
-See the L<BUGS> section for details.
+See the L<LIMITATIONS> section for details.
 
 =head2 Incompatibilities with previous versions of C<Curses>
 
-In previous versions of this software, some Perl functions took a
-different set of parameters than their C counterparts.  This is no
-longer true.  You should now use C<getstr($str)> and C<getyx($y, $x)>
-instead of C<$str = getstr()> and C<($y, $x) = getyx()>.
-
-=head2 Incompatibilities with other Perl programs
-
-    menu.pl, v3.0 and v3.1
-	There were various interaction problems between these two
-	releases and Curses.  Please upgrade to the latest version
-	(v3.3 as of 3/16/96).
+In previous versions of this software, some Perl functions took a different
+set of parameters than their C counterparts.  This is not true in the current
+version.  You should now use C<getstr($str)> and C<getyx($y, $x)> instead of
+C<$str = getstr()> and C<($y, $x) = getyx()>.
 
 =head1 DIAGNOSTICS
 
@@ -396,34 +542,37 @@ You have called a C<Curses> function with a wrong number of
 arguments.
 
 =item * argument %d to Curses function '%s' is not a Curses %s at ...
+
 =item * argument is not a Curses %s at ...
 
-The argument you gave to the function wasn't what it wanted.
+The argument you gave to the function wasn't of a valid type for the
+place you used it.
 
-This probably means that you didn't give the right arguments to a
-I<unified> function.  See the DESCRIPTION section on L<Unified
-Functions> for more information.
+This probably means that you didn't give the right arguments to a I<unified>
+function.  See the DESCRIPTION section on L<Unified Functions> for more
+information.
 
 =item * Curses function '%s' is not defined by your vendor at ...
 
-You have a C<Curses> function in your code that your system's curses(3)
-library doesn't define.
+Your code has a call to a Perl C<Curses> function that your system's Curses
+library doesn't provide.
 
 =item * Curses variable '%s' is not defined by your vendor at ...
 
-You have a C<Curses> variable in your code that your system's curses(3)
-library doesn't define.
+Your code has a Perl C<Curses> variable that your system's Curses library
+doesn't provide.
 
 =item * Curses constant '%s' is not defined by your vendor at ...
 
-You have a C<Curses> constant in your code that your system's curses(3)
-library doesn't define.
+Your code has a C<Curses> constant that your system's Curses library doesn't
+provide.
 
 =item * Curses::Vars::FETCH called with bad index at ...
+
 =item * Curses::Vars::STORE called with bad index at ...
 
-You've been playing with the C<tie> interface to the C<Curses>
-variables.  Don't do that.  :-)
+You've been playing with the C<tie> interface to the C<Curses> variables.
+Don't do that.  :-)
 
 =item * Anything else
 
@@ -431,7 +580,7 @@ Check out the F<perldiag> man page to see if the error is in there.
 
 =back
 
-=head1 BUGS
+=head1 LIMITATIONS
 
 If you use the variables C<$stdscr> and C<$curscr> instead of their
 functional counterparts (C<stdscr> and C<curscr>), you might run into
@@ -440,8 +589,6 @@ manifested by the C<Curses> package telling you C<$stdscr> isn't a
 window.  One workaround is to put a line like C<$stdscr = $stdscr>
 near the front of your program.
 
-Probably many more.
-
 =head1 AUTHOR
 
 William Setzer <William_Setzer@ncsu.edu>
@@ -773,11 +920,22 @@ William Setzer <William_Setzer@ncsu.edu>
     form_request_name       No
     form_request_by_name    No
 
-[*] To use any functions in this column, the variable
-C<$Curses::OldCurses> must be set to a non-zero value before using the
+[*] To use any functions in this column, the program must set the variable
+C<$Curses::OldCurses> variable to a non-zero value before using the
 C<Curses> package.  See L<"Perl 4.X cursperl Compatibility"> for an
 example of this.
 
+=head2 Available Wide-Character-Aware Functions
+
+    Function    Uses wide-character call  Reverts to legacy call
+    --------    ------------------------  ----------------------
+    getchar     wget_wch                  wgetch
+    getstring   wgetn_wstr                wgetnstr
+    ungetchar   unget_wch                 ungetch
+    instring    winnwtr                   winnstr
+    addstring   waddnwstr                 waddnstr
+    insstring   wins_nwstr                winsnstr
+
 =head2 Available Variables
 
     LINES                   COLS                    stdscr
@@ -884,19 +1042,19 @@ example of this.
     O_PASSOK                O_STATIC                O_NL_OVERLOAD
     O_BS_OVERLOAD           
 
-=head2 curses(3) functions not available through C<Curses>
+=head2 Curses functions not available through Perl C<Curses>
 
     tstp _putchar fullname scanw wscanw mvscanw mvwscanw ripoffline
     setupterm setterm set_curterm del_curterm restartterm tparm tputs
     putp vidputs vidattr mvcur tigetflag tigetnum tigetstr tgetent
     tgetflag tgetnum tgetstr tgoto tputs
 
-=head2 menu(3) functions not available through C<Curses>
+=head2 Curses menu functions not available through Perl C<Curses>
 
     set_item_init item_init set_item_term item_term set_menu_init
     menu_init set_menu_term menu_term
 
-=head2 form(3) functions not available through C<Curses>
+=head2 Curses form functions not available through Perl C<Curses>
 
     new_fieldtype free_fieldtype set_fieldtype_arg
     set_fieldtype_choice link_fieldtype set_form_init form_init
