@@ -9,21 +9,18 @@
 
 #include <ncurses.h>
 
-#ifdef C_PANELSUPPORT
+#ifdef C_PANELFUNCTION
 #include <panel.h>
 #endif
 
-#ifdef C_MENUSUPPORT
+#ifdef C_MENUFUNCTION
 #include <menu.h>
 #endif
 
-#ifdef C_FORMSUPPORT
+#ifdef C_FORMFUNCTION
 #include <form.h>
 #endif
 
-/* Defined in both Perl & Ncurses */
-#undef instr
-
 #define C_LONGNAME
 #define C_LONG0ARGS
 #undef  C_LONG2ARGS
